/*
Another idea:
  Workers -> Sites
  Processes -> Agents
*/
var stream = require('stream');
var fs = require('fs');
var yaml = require('js-yaml');

var Pubsub = require('./Pubsub.js');
var Code = require('./Code.js');
var helpers = require('../helpers.js');

/*
Properties of Sites/Agents:
    - CyberPhysical - means it has physical interface. This has some implications:
        - cannot be migrated
        - incoming data is external (security)
    - Stateful - This has some implications:
        - code must be instrumented
    - hasIOLoop - means it is a non terminating program
 */

/* This is a "Client-side" object - meaning this object does not spawn a process itself;
 *   it will make a Pubsub request to spawn processes.
 */
function Agent(name){
	this.id = helpers.randKey();
	this.name = name;
	this.code_name = name+'.js';
	this.config = {
		pipes: []
	}

	// this.duties = duties;
	// this.agents = {} // nested agents?
	
	// this.stdin // general input (Agent's inbox)
	// this.stdout // general output (Agent's twitter)
	
	this.plan = [];

	// need to extract from duties
	// what information it subscribes to and publishes to
	// need to characterize the agent, whether bound to a site or not
	// 
}
Agent.prototype.updatePlan = function(action, args){
	this.plan.push({ action: action, args: args })
	Agent.ACTIONS[action](this, args);
}
Agent.prototype.dispatch = function(){
	console.log(this.config);
}
// Agent.prototype.searchSite = function(sites){

// }
// Agent.prototype.validateSite = function(site){

// }

// Agent.prototype.beginWork = function(site){
// 	console.log(this.plan);	
// }
// Agent.prototype.endWork = function(site){
	
// }
Agent.ACTIONS = {
	'listenTo': function(self, agent){
		// self.config.pipes.push({ topic: 'proc/'+agent.id+'/stdout', type: 'input' });
		self.config.pipes.push({ topic_from: 'proc/'+agent.id+'/stdout', topic_to: 'proc/'+self.id+'/stdin' });
	},
	'speakTo': function(self, agent){
		// self.config.pipes.push({ topic: 'proc/'+agent.id+'/stdin', type: 'output' });
		self.config.pipes.push({ topic_from: 'proc/'+self.id+'/stdout', topic_to: 'proc/'+agent.id+'/stdin' });
	}
}


function Application(agents, plan){
	this.agents = agents;
	this.plan = plan;

	this.compile();
}
Application.prototype.compile = function(){
	var self = this;
	this.plan.forEach(function(statement){
		// statement = { agent_name, func_name, func_args }
		console.log(statement);
		self.agents[statement.agent_name].updatePlan(statement.func_name, statement.func_args);
	});
	Object.keys(this.agents).forEach(function(name){
		var agent = self.agents[name];
		// dispatching - this should be done through "scheduler" probably
		// something like agent.searchSite(sites)
		// check site against constraints -> agent.validateSite(site)
		// agent.beginWork(site);
	});
}
Application.prototype.dispatch = function(scheduler){
	if (scheduler){

	}
	else {
		Object.values(this.agents)
			.forEach(function(agent){
				agent.dispatch();
			})
	}
}
Application.prototype.pause = function(){

}
Application.prototype.resume = function(){

}

Application.fromCode = function(code){
	// parse the code and build the application
	var program = yaml.safeLoad(code);

	console.log(program);

	// parse agents
	program.agents = program.agents.reduce(function(acc, agent_name){
		acc[agent_name] = new Agent(agent_name);
		return acc;
	}, {});

	// parse plans
	program.plan = program.plan.map(function(str){
		var statement = {};
		var tokens = str.split('.');
		statement.agent_name = tokens[0];
		tokens = tokens[1].split('(');
		statement.func_name = tokens[0];
		tokens = tokens[1].split(')');
		statement.func_args = program.agents[tokens[0]];
		return statement;
	})

	return new Application(program.agents, program.plan);
}
Application.fromFile = function(file_path){
	return Application.fromCode(fs.readFileSync(file_path).toString());
}

module.exports = Application;