var EventEmitter = require('events').EventEmitter;
var Pubsub = require('./Pubsub.js');
var helpers = require('../helpers.js');

var ENGINE_REGISTRY_NAMESPACE = 'engine-registry';
var PROGRAM_MONITOR_NAMESPACE = 'program-monitor';

function Thing(config){
	if (!(this instanceof Thing)) return new Thing(config);
	EventEmitter.call(this);
	var self = this;
	this.config = Object.assign({
		pubsub_url: 'mqtt://localhost'
	}, config);
	this.id = (this.config.id || 'thing-'+helpers.randKey());
	// this.options = Object.assign({}, options);
	// console.log(chalk.green('[Thing:'+this.id+'] Initialized'));

	this.pubsub = new Pubsub(this.config.pubsub_url);
	
	this.engines = {};
	this.programs = {};

	this._requests = {};

	this.pubsub.subscribe(ENGINE_REGISTRY_NAMESPACE, function(message, topic){
		if (!(message.id in self.engines)){
			self.engines[message.id] = new Engine(self, message.id, message.meta);
		}
		self.engines[message.id].status = message.status;
		self.engines[message.id].meta = message.meta;
		self.engines[message.id].codes = message.codes;
	});

	this.pubsub.subscribe(PROGRAM_MONITOR_NAMESPACE, function(message, topic){
		if (!(message.instance_id in self.programs)){
			self.programs[message.instance_id] = new Program(self, message.code_name, message.instance_id, message.source);
		}
		self.programs[message.instance_id].status = message.status;
		if (message.source) self.programs[message.instance_id].source = message.source;
	});

	this.pubsub.subscribe(this.id+'/cmd', function(message){
		if (message.ctrl in self.constructor.Behaviours){
			self.constructor.Behaviours[message.ctrl](self, message.kwargs)
				.then(function(result){
					// console.log(result);
					// self.reportStatus();

					if (message.request_id && message.reply_to){
						self.pubsub.publish(message.reply_to, {
							reply_id: message.request_id,
							payload: result
						})
					}
				}, function(err){
					console.log(err);
				});
		}
		else if (message.reply_id){
			self._requests[message.reply_id].resolve(message.payload);
			clearTimeout(self._requests[message.reply_id].timer);
			delete self._requests[message.reply_id];
		}
		else {
			console.log(chalk.red('[Thing:'+self.id+'] Received unexpected message'));
		}
	});

	this.pubsub.on('ready', function(){
		self.emit('ready');
		console.log(chalk.green('[Thing:'+self.id+'] connected to Pub/Sub at '+self.pubsub.url));
		self.requestStatusReports();
	});
}
Thing.prototype.requestStatusReports = function(){
	this.pubsub.publish(ENGINE_REGISTRY_NAMESPACE+'/bcast', { ctrl: 'report' });
};

Thing.prototype.ackedPublish = function(topic, ctrl, kwargs){
	var self = this;
	var deferred = helpers.defer();
	var request_id = helpers.randKey(16);
	self._requests[request_id] = deferred;

	self.pubsub.publish(topic, {
		request_id: request_id,
		reply_to: self.id+'/cmd',
		ctrl: ctrl,
		kwargs: kwargs
	})
	deferred.timer = setTimeout(function(){
		if (request_id in self._requests){
			deferred.reject('PubsubCommandTimeout');
			delete self._requests[request_id];
		}
	}, 10000); // assume failure if reply not received
	return deferred.promise
}

module.exports = Thing